/*
    Copyright (C) 2006  Robert Nelson

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#define sstModule          0x120
#define sstGlobalPub       0x12a
#define sstSegMap          0x12d

typedef struct CVSignature {
    CHAR          Signature[4];  // "NBxx"
    INT           filepos;       // offset in file
} CVSignature;

typedef struct CVDirHeader {
    USHORT        cbDirHeader;   // length of this structure
    USHORT        cbDirEntry;    // number of bytes in each directory entry
    UINT          cDir;          // number of directorie entries
    INT           lfoNextDir;    // offset from base of next directory
    UINT          flags;         // status flags
} CVDirHeader;

typedef struct CVDirEntry {
    USHORT        SubSection;    // subsection type (sst...)
    USHORT        iMod;          // module index
    INT           lfo;           // large file offset of subsection
    UINT          cb;            // number of bytes in subsection
} CVDirEntry;

typedef struct CVSegDesc {
    USHORT        Seg;           // segment index
    USHORT        pad;           // pad to maintain alignment
    UINT          Off;           // offset of code in segment
    UINT          cbSeg;         // number of bytes in segment
} CVSegDesc;

typedef struct CVModule {
    USHORT        ovlNumber;     // overlay number
    USHORT        iLib;          // library that the module was linked from
    USHORT        cSeg;          // count of number of segments in module
    CHAR          Style[2];      // debugging style "CV"
    CVSegDesc     SegInfo[1];    // describes segments in module
    CHAR          Name[];        // length prefixed module name padded to
                                 // long word boundary
} CVModule;

typedef struct CVSegMapDesc {
    USHORT        flags;         // descriptor flags bit field.
    USHORT        ovl;           // the logical overlay number
    USHORT        group;         // group index into the descriptor array
    USHORT        frame;         // logical segment index - interpreted via flags
    USHORT        iSegName;      // segment or group name - index into sstSegName
    USHORT        iClassName;    // class name - index into sstSegName
    UINT          offset;        // byte offset of the logical within the physical segment
    UINT          cbSeg;         // byte count of the logical segment or group
} CVSegMapDesc;

typedef struct CVSegMap {
    USHORT        cSeg;          // total number of segment descriptors
    USHORT        cSegLog;       // number of logical segment descriptors
    CVSegMapDesc  rgDesc[0];     // array of segment descriptors
} CVSegMap;

typedef struct CVSymHash {
    USHORT        symhash;       // symbol hash function index
    USHORT        addrhash;      // address hash function index
    UINT          cbSymbol;      // length of symbol information
    UINT          cbHSym;        // length of symbol hash data
    UINT          cbHAddr;       // length of address hashdata
} CVSymHash;

#define S_PUB32   0x0203         // a public symbol

typedef struct DATASYM32 {
    USHORT        reclen;        // Record length
    USHORT        rectyp;        // S_LDATA32, S_GDATA32 or S_PUB32
    UINT          off;
    USHORT        seg;
    USHORT        typind;        // Type index
    UCHAR         name[1];       // Length-prefixed name
} DATASYM32;

typedef DATASYM32 PUBSYM32;
